(* Drucken von PDF-Dateien mit Sumatra PDF
   =======================================

   2006-2017  Dr. J. Rathlev, D-24222 Schwentinental

   -----------------------------------------------------------------------------
   The contents of this file may be used under the terms of the
   Mozilla Public License ("MPL") or
   GNU Lesser General Public License Version 2 or later (the "LGPL")

   Software distributed under this License is distributed on an "AS IS" basis,
   WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
   the specific language governing rights and limitations under the License.

   Vers. 1,   Mar. 2006 : mit Adobe Reader 
   Vers. 2,   Nov. 2009 : mit Foxit Reader
   Vers. 3,   Jan. 2012 : mit Delphi XE2
         3.1, March 2017 (Delphi 10)
         3.2, Mrch 2019: new printer selection
         3.3, Feb. 2023: use Sumatra as PDF-Viewer, Unicode Ini file
   *)

unit SPdfPrint;

interface

uses
  Winapi.Windows, Winapi.Messages, System.SysUtils, System.Classes, Vcl.Graphics,
  Vcl.Controls, Vcl.Forms, Vcl.Dialogs, Vcl.StdCtrls, Vcl.Buttons,
  WinUtils, WinShell, Vcl.ComCtrls, Vcl.Samples.Spin, Vcl.ExtCtrls,
  Vcl.Printers, WinApiUtils, System.ImageList, Vcl.ImgList, NumberEd;

const
  ProgName = 'Ausdruck von PDF-Dateien';
  Vers = ' (Vers. 3.3)';
  CopRgt = ' 2006-2023 Dr. J. Rathlev, D-24222 Schwentinental';
  EmailAdr = 'kontakt(a)rathlev-home.de';

  BatExt = 'pbt';       // batch file list - Format: Filename

  SumatraPath ='SumatraPDF\';
  SumatraExe = 'PDF.exe';

type
  THauptForm = class(TForm)
    Label1: TLabel;
    Label2: TLabel;
    EndeBtn: TBitBtn;
    OpenDialog: TOpenDialog;
    SourceSelectBtn: TSpeedButton;
    FileBtn: TSpeedButton;
    chbSubDir: TCheckBox;
    btnProcess: TBitBtn;
    PrinterSetupDialog: TPrinterSetupDialog;
    Label3: TLabel;
    KopAnz: TSpinEdit;
    btnEdit: TBitBtn;
    btnNew: TBitBtn;
    gbxKop: TGroupBox;
    Label4: TLabel;
    edtKop: TNumberEdit;
    cxPrinter: TComboBox;
    StatusBar: TStatusBar;
    Label5: TLabel;
    ilPrint: TImageList;
    SourceDirEdit: TComboBox;
    FileEdit: TComboBox;
    InfoBtn: TBitBtn;
    procedure FormCreate(Sender: TObject);
    procedure EndeBtnClick(Sender: TObject);
    procedure InfoBtnClick(Sender: TObject);
    procedure FileBtnClick(Sender: TObject);
    procedure btnProcessClick(Sender: TObject);
    procedure SourceSelectBtnClick(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure FileEditChange(Sender: TObject);
    procedure btnEditClick(Sender: TObject);
    procedure btnNewClick(Sender: TObject);
    procedure cxPrinterCloseUp(Sender: TObject);
  private
    { Private-Deklarationen }
    ProgVersName,
    ProgVersDate,
    AppPath,UserPath,
    ProgPath,IniName,
    PrtName,SumatraPath : string;
    Printing,Stopped    : boolean;
  public
    { Public-Deklarationen }
  end;

var
  HauptForm: THauptForm;

implementation

{$R *.DFM}

uses ShellDirDlg, Winapi.ShlObj, WinExecute, System.StrUtils, FileListDlg, MsgDialogs,
  System.IniFiles, IniFileUtils, PathUtils, StringUtils, LangUtils, GnuGetText, InitProg;

const
  IniExt = 'ini';

  (* INI-Sektionen *)
  CfGSekt = 'Config';
  SourceSekt = 'Source';
  DestSekt = 'Dest';
  FileSekt = 'Files';

  (* INI-Variablen *)
  IniLeft= 'Left';
  IniTop = 'Top';
  iniSubDir = 'Subdirectories';
  iniPrtName = 'Printer';
  iniSumatra = 'Sumatra';

{ ------------------------------------------------------------------- }
(* INI-File lesen *)
procedure THauptForm.FormCreate(Sender: TObject);
var
  IniFile  : TUnicodeIniFile;
  s        : string;
begin
  TranslateComponent(self);
  InitPaths(AppPath,UserPath,ProgPath);
  InitVersion(ProgName,Vers,CopRgt,2,2,ProgVersName,ProgVersDate);
  Caption:='PDF Print - '+VersInfo.Comments;
  IniName:=Erweiter(AppPath,PrgName,IniExt);
  IniFile:=TUnicodeIniFile.CreateForRead(IniName);
  with IniFile do begin
    Left:=ReadInteger (CfgSekt,IniLeft,Left);
    Top:=ReadInteger (CfgSekt,IniTop,Top);
    SumatraPath:=ReadString(CfgSekt,IniSumatra,'');
    PrtName:=ReadString(CfgSekt,IniPrtName,'');
    chbSubDir.Checked:=ReadBool(CfgSekt,iniSubDir,false);
    Free;
    end;
  LoadHistory(IniName,SourceSekt,SourceDirEdit);
  SourceDirEdit.ItemIndex:=0;
  LoadHistory(IniName,FileSekt,FileEdit);
  FileEdit.ItemIndex:=0;
  if Paramcount>0 then begin
    s:=ExtractFilepath(ParamStr(1));
    with SourceDirEdit do begin
      Text:=s; AddToHistory(Items,s);
      end;
    s:=ExtractFilename(ParamStr(1));
    with FileEdit do begin
      Text:=s; AddToHistory(Items,s);
      end;
    end;
  Printing:=false; Stopped:=false;
  StatusBar.SimpleText:='';
  with Printer.Printers do begin
    if Count>0 then cxPrinter.Items:=Printer.Printers;
    end;
  end;

procedure THauptForm.FormDestroy(Sender: TObject);
var
  IniFile  : TUnicodeIniFile;
begin
  IniFile:=TUnicodeIniFile.CreateForWrite(IniName);
  with IniFile do begin
    WriteInteger(CfgSekt,IniLeft,Left);
    WriteInteger(CfgSekt,IniTop,Top);
    WriteString(CfgSekt,IniSumatra,SumatraPath);
    WriteString(CfgSekt,IniPrtName,PrtName);
    WriteBool(CfgSekt,iniSubDir,chbSubDir.Checked);
    Free;
    end;
  SaveHistory(IniName,SourceSekt,true,SourceDirEdit);
  SaveHistory(IniName,FileSekt,true,FileEdit);
  end;

procedure THauptForm.FormShow(Sender: TObject);
var
  ok : boolean;
begin
  with Printer do if Printers.Count>0 then begin
    PrinterIndex:=Printers.IndexOf(PrtName);
    PrtName:=Printers[PrinterIndex];
    with cxPrinter do ItemIndex:=Items.IndexOf(PrtName);
    end
  else begin
    ErrorDialog(Caption,_('No printer available!'));
    Close;
    end;
  if (length(SumatraPath)=0) or not FileExists(SumatraPath) then begin
    SumatraPath:=PrgPath+SumatraExe;   // check PdfPrint path
    ok:=FileExists(SumatraPath);
    if not ok then begin  // check for 32-bit installation
      SumatraPath:=IncludeTrailingPathDelimiter(GetProgramFolder(pfProgramFiles86))+SumatraPath+SumatraExe;
      ok:=FileExists(SumatraPath);
      end;
    if not ok then begin  // check for 64-bit installation
      SumatraPath:=IncludeTrailingPathDelimiter(GetProgramFolder(pfProgramFiles64))+SumatraPath+SumatraExe;
      ok:=FileExists(SumatraPath);
      end;
    if not ok then begin  // check for local installation
      SumatraPath:=IncludeTrailingPathDelimiter(GetDesktopFolder(CSIDL_LOCAL_APPDATA))+SumatraPath+SumatraExe;
      ok:=FileExists(SumatraPath);
      end;
    if not ok then begin
      with OpenDialog do begin
        InitialDir:=GetDesktopFolder(CSIDL_PROGRAM_FILES);
        Filename:='';
        Filter:=_('Programs|*.exe'); //'Programme|*.exe'
        Title:=_('Seek for Sumatra PDF');   //'Sumatra PDF suchen'
        if Execute then SumatraPath:=Filename
        else begin
          //'Sumatra PDF nicht gefunden!'+sLineBreak+'Kein Ausdruck von PDF-Dateien mglich!
          ErrorDialog(Caption,_('Sumatra PDF not found on this computer!'+sLineBreak
            +'Printing of PDF files using this program is not available!'));
          Close;
          end;
        end;
      end;
    end;
  with btnProcess do begin
    Glyph:=nil;
    ilPrint.GetBitmap(1,Glyph);
    Caption:=_('Print');        //'Drucken';
    Hint:=_('Start printing');  //'Ausdruck starten'
    end;
  FileEditChange(Sender);
  end;

{ ------------------------------------------------------------------- }
(* Programmende *)
procedure THauptForm.EndeBtnClick(Sender: TObject);
begin
  Close;
  end;

{ ------------------------------------------------------------------- }
procedure THauptForm.InfoBtnClick(Sender: TObject);
begin
  InfoDialog(CursorPos,ProgVersName+' - '+ProgVersDate+sLineBreak+CopRgt
           +sLineBreak+'E-Mail: '+EmailAdr+sLineBreak+
           _('Installation of Sumatra PDF required:')+sLineBreak+
           'https://www.sumatrapdfreader.org/free-pdf-reader');
  end;

{ ------------------------------------------------------------------- }
procedure THauptForm.SourceSelectBtnClick(Sender: TObject);
var
  s : string;
begin
  s:=SourceDirEdit.Text;
  if length(s)=0 then s:=UserPath;
  // 'Verzeichnis mit den zu druckenden Dateien'
  if ShellDirDialog.Execute (_('Directory with files to print'),true,true,false,'',s) then begin
    with SourceDirEdit do begin
      Text:=s; AddToHistory(Items,s);
      end;
    end;
  end;

procedure THauptForm.FileBtnClick(Sender: TObject);
var
  i : integer;
  s : string;
begin
  with OpenDialog do begin
    InitialDir:=SourceDirEdit.Text;
    Filename:='';
    //'PDF-Dateien|*.pdf|Drucklisten|*.%s'
    Filter:=Format(_('PDF files|*.pdf|Print list|*.%s'),[BatExt]);
    Title:=_('Select files to print or print list'); //'Zu druckende Dateien auswhlen'
    if Execute then begin
      chbSubDir.Checked:=false;
      s:=ExtractFilePath(Files.Strings[0]);
      with SourceDirEdit do if s<>Text then begin
        Text:=s; AddToHistory(Items,s);
        end;
      with Files do begin
        s:='';
        for i:=0 to Count-1 do s:=s+ExtractFilename(Strings[i])+';';
        end;
      delete (s,length(s),1);
      with FileEdit do begin
        Text:=s; AddToHistory(Items,s);
        end;
      end;
    end;
  end;

procedure THauptForm.btnProcessClick(Sender: TObject);
var
  FListe        : TStringList;
  fcnt,i,j      : integer;
  bf            : TextFile;
  s,t,fname     : string;

  procedure ScanDir (Dir,Mask : string);
  var
    DirInfo    : TSearchRec;
    Findresult : integer;
  begin
    if chbSubDir.Checked then begin
      FindResult:=FindFirst (Erweiter(Dir,'*','*'),faAnyFile,DirInfo);
      while FindResult=0 do with DirInfo do begin
        if NotSpecialDir(Name) then begin
          if((Attr and faDirectory)<>0) then ScanDir (Erweiter(Dir,Name,''),Mask);
          end;
        FindResult:=FindNext (DirInfo);
        end;
      FindClose(DirInfo);
      end;
    FindResult:=FindFirst (Erweiter(Dir,Mask,''),faArchive+faReadOnly+faHidden+faSysfile,DirInfo);
    while FindResult=0 do with DirInfo do begin
      if NotSpecialDir(Name) then FListe.Add(Erweiter(Dir,Name,''));
      FindResult:=FindNext (DirInfo);
      end;
    FindClose(DirInfo);
    end;

begin
  if Printing then Stopped:=true
  else begin
    cxPrinter.Enabled:=false;
    t:=FileEdit.Text;
    if (length(t)>0) then begin
      Printing:=true;
      with StatusBar do begin
        Font.Color:=clBlack;
        SimpleText:='';
        end;
      FListe:=TStringList.Create;
      fcnt:=0;
      with btnProcess do begin
        Glyph:=nil;
        ilPrint.GetBitmap(0,Glyph);
        Caption:=_('Stop');  // 'Stopp'
        Hint:=_('Stop printing');  //'Ausdruck anhalten'
        end;
      FListe.Clear;
      repeat
        s:=ReadNxtStr (t,';');
        if LowerCase(GetExt(s))=BatExt then begin // get files from batch list
          s:=Erweiter(SourceDirEdit.Text,s,'');
          if FileExists (s) then begin
            AssignFile(bf,s); reset(bf);
            while not Eof(bf) do begin
              readln (bf,s);
              FListe.Add(Erweiter(SourceDirEdit.Text,s,''));
              end;
            CloseFile(bf);
            end
          else ErrorDialog(Caption,Format(_('Print list: %s not found!'),[s]));  //'Druckliste: %s nicht gefunden!'
          end
        else ScanDir (SourceDirEdit.Text,s);
        until (length(t)=0);
      for j:=1 to KopAnz.Value do begin
        edtKop.Value:=j;
        with FListe do if Count>0 then for i:=0 to Count-1 do begin
          if Stopped then Break;
          fName:=Strings[i];
          StatusBar.SimpleText:=Format(_('File: %s'),[StripPath(fName,50)]);
          s:=AnsiQuotedStr(SumatraPath,'"')+' -print-to '+AnsiQuotedStr(PrtName,'"')+Space+AnsiQuotedStr(fName,'"');
          if ExecuteProcess (s,SourceDirEdit.Text)<0 then begin
            if MessageDialog (Caption,Format(_('File: %s'+sLineBreak+'could not be printed!'),[fName]),
                  mtError,[mbIgnore,mbCancel])=mrCancel then Stopped:=true;
            end
          else begin
            inc(fcnt);
            Sleep(1000);  // kleine Pause
            end;
          end;
        if Stopped then Break;
        end;
      FListe.Free;
      end;
    with StatusBar do begin
      Font.Color:=clBlue;
      if fcnt=0 then SimpleText:=Space+_('No file printed')        //' Es wurden keine Dateien gedruckt'
      else if fcnt=1 then SimpleText:=Space+_('1 file printed')    //' Es wurde 1 Datei gedruckt'
      else SimpleText:=Format(Space+_('%u files printed'),[fcnt]); //' Es wurden %u Dateien gedruckt'
      end;
    with btnProcess do begin
      Glyph:=nil;
      ilPrint.GetBitmap(1,Glyph);
      Caption:=_('Print');        //'Drucken';
      Hint:=_('Start printing');  //'Ausdruck starten'
      end;
    Printing:=false; Stopped:=false;
    cxPrinter.Enabled:=true;
    end;
  end;

procedure THauptForm.cxPrinterCloseUp(Sender: TObject);
begin
  with Printer do begin
    PrtName:=cxPrinter.Text;
    PrinterIndex:=Printers.IndexOf(PrtName);
    cxPrinter.ItemIndex:=PrinterIndex;
    end;
  end;

procedure THauptForm.FileEditChange(Sender: TObject);
begin
  chbSubDir.Enabled:=AnsiContainsText(FileEdit.Text,'*') or
                     AnsiContainsText(FileEdit.Text,'?');
  end;

procedure THauptForm.btnEditClick(Sender: TObject);
var
  s : string;
begin
  with OpenDialog do begin
    InitialDir:=SourceDirEdit.Text;
    if AnsiContainsText(FileEdit.Text,BatExt) then
      Filename:=Copy(FileEdit.Text,1,AnsiPos(BatExt,FileEdit.Text)+3)
    else Filename:='';
    Filter:=Format(_('Print list|*.%s'),[BatExt]);  //'Druckliste|*.%s'
    Title:=_('Select print list');    //'Druckliste auswhlen'
    if Execute then begin
      s:=FileListDialog.Execute(_('Edit print list'),_('PDF files|*.pdf'),   //'Druckliste bearbeiten' 'PDF-Dateien|*.pdf'
        SourceDirEdit.Text,Filename,Filter,BatExt,true);
      if length(s)>0 then begin
        FileEdit.Text:=ExtractFilename(s);
        s:=ExtractFilePath(s);
        with SourceDirEdit do if s<>Text then begin
          Text:=s; AddToHistory(Items,s);
          end;
        end;
      end;
    end;
  end;

procedure THauptForm.btnNewClick(Sender: TObject);
var
  s : string;
begin
  s:=FileListDialog.Execute(_('Create print list'),_('PDF files|*.pdf'),   //'Druckliste erstellen'
       SourceDirEdit.Text,'',Format(_('Batch files|*.%s'),[BatExt]),BatExt,true);
  if length(s)>0 then begin
    FileEdit.Text:=ExtractFilename(s);
    s:=ExtractFilePath(s);
    with SourceDirEdit do if s<>Text then begin
      Text:=s; AddToHistory(Items,s);
      end;
    end;
  end;

end.

